@@ -24,6 +24,9 @@ class ModelArgs:
     adam_epsilon: float = 1e-8
     best_model_dir: str = "outputs/best_model"
     cache_dir: str = "cache_dir/"
+    custom_layer_parameters: list = field(default_factory=list)
+    custom_parameter_groups: list = field(default_factory=list)
+    train_custom_parameters_only: bool = False
     config: dict = field(default_factory=dict)
     do_lower_case: bool = False
     early_stopping_consider_epochs: bool = False
@@ -100,24 +103,25 @@ class ClassificationArgs(ModelArgs):
     Model args for a ClassificationModel
     """
 
-    tie_value: int = 1
-    stride: float = 0.8
-    sliding_window: bool = False
-    regression: bool = False
-    lazy_text_column: int = 0
-    lazy_text_b_column: bool = None
-    lazy_text_a_column: bool = None
-    lazy_labels_column: int = 1
-    lazy_header_row: bool = True
-    lazy_delimiter: str = "\t"
     labels_list: list = field(default_factory=list)
     labels_map: dict = field(default_factory=dict)
+    lazy_delimiter: str = "\t"
+    lazy_labels_column: int = 1
+    lazy_loading: bool = False
+    lazy_loading_start_line: int = 1
+    lazy_text_a_column: bool = None
+    lazy_text_b_column: bool = None
+    lazy_text_column: int = 0
+    regression: bool = False
+    sliding_window: bool = False
+    stride: float = 0.8
+    tie_value: int = 1
 
 
 @dataclass
 class MultiLabelClassificationArgs(ModelArgs):
     """
-    Model args for a ClassificationModel
+    Model args for a MultiLabelClassificationModel
     """
 
     sliding_window: bool = False
@@ -126,6 +130,7 @@ class MultiLabelClassificationArgs(ModelArgs):
     tie_value: int = 1
     labels_list: list = field(default_factory=list)
     labels_map: dict = field(default_factory=dict)
+    lazy_loading: bool = False
 
 
 @dataclass
@@ -136,6 +141,8 @@ class NERArgs(ModelArgs):
 
     classification_report: bool = False
     labels_list: list = field(default_factory=list)
+    lazy_loading: bool = False
+    lazy_loading_start_line: int = 0
 
 
 @dataclass
@@ -198,6 +205,9 @@ class LanguageModelingArgs(ModelArgs):
     tie_generator_and_discriminator_embeddings: bool = True
     tokenizer_name: str = None
     vocab_size: int = None
+    clean_text: bool = True
+    handle_chinese_chars: bool = True
+    strip_accents: bool = True
     local_rank: int = -1
 
 
@@ -220,7 +230,7 @@ class Seq2SeqArgs(ModelArgs):
     repetition_penalty: float = 1.0
     top_k: float = None
     top_p: float = None
-    use_multiprocessed_decoding: bool = True
+    use_multiprocessed_decoding: bool = False
 
 
 @dataclass
