@@ -127,13 +127,6 @@ class LanguageModelingModel:
             **kwargs (optional): For providing proxies, force_download, resume_download, cache_dir and other options specific to the 'from_pretrained' implementation where this will be supplied.
         """  # noqa: ignore flake8"
 
-        if args and "manual_seed" in args:
-            random.seed(args.manual_seed)
-            np.random.seed(args.manual_seed)
-            torch.manual_seed(args.manual_seed)
-            if "n_gpu" in args and args.n_gpu > 0:
-                torch.cuda.manual_seed_all(args.manual_seed)
-
         self.args = self._load_model_args(model_name)
 
         if isinstance(args, dict):
@@ -427,13 +420,58 @@ class LanguageModelingModel:
             t_total = len(train_dataloader) // args.gradient_accumulation_steps * args.num_train_epochs
 
         no_decay = ["bias", "LayerNorm.weight"]
-        optimizer_grouped_parameters = [
-            {
-                "params": [p for n, p in model.named_parameters() if not any(nd in n for nd in no_decay)],
-                "weight_decay": args.weight_decay,
-            },
-            {"params": [p for n, p in model.named_parameters() if any(nd in n for nd in no_decay)]},
-        ]
+
+        optimizer_grouped_parameters = []
+        custom_parameter_names = set()
+        for group in self.args.custom_parameter_groups:
+            params = group.pop("params")
+            custom_parameter_names.update(params)
+            param_group = {**group}
+            param_group["params"] = [p for n, p in model.named_parameters() if n in params]
+            optimizer_grouped_parameters.append(param_group)
+
+        for group in self.args.custom_layer_parameters:
+            layer_number = group.pop("layer")
+            layer = f"layer.{layer_number}."
+            group_d = {**group}
+            group_nd = {**group}
+            group_nd["weight_decay"] = 0.0
+            params_d = []
+            params_nd = []
+            for n, p in model.named_parameters():
+                if n not in custom_parameter_names and layer in n:
+                    if any(nd in n for nd in no_decay):
+                        params_nd.append(p)
+                    else:
+                        params_d.append(p)
+                    custom_parameter_names.add(n)
+            group_d["params"] = params_d
+            group_nd["params"] = params_nd
+
+            optimizer_grouped_parameters.append(group_d)
+            optimizer_grouped_parameters.append(group_nd)
+
+        if not self.args.train_custom_parameters_only:
+            optimizer_grouped_parameters.extend(
+                [
+                    {
+                        "params": [
+                            p
+                            for n, p in model.named_parameters()
+                            if n not in custom_parameter_names and not any(nd in n for nd in no_decay)
+                        ],
+                        "weight_decay": args.weight_decay,
+                    },
+                    {
+                        "params": [
+                            p
+                            for n, p in model.named_parameters()
+                            if n not in custom_parameter_names and any(nd in n for nd in no_decay)
+                        ],
+                        "weight_decay": 0.0,
+                    },
+                ]
+            )
 
         warmup_steps = math.ceil(t_total * args.warmup_ratio)
         args.warmup_steps = warmup_steps if args.warmup_steps == 0 else args.warmup_steps
@@ -517,9 +555,13 @@ class LanguageModelingModel:
                 epochs_trained -= 1
                 continue
             train_iterator.set_description(f"Epoch {epoch_number + 1} of {args.num_train_epochs}")
-            for step, batch in enumerate(
-                tqdm(train_dataloader, desc=f"Running Epoch {epoch_number}", disable=args.silent)
-            ):
+            batch_iterator = tqdm(
+                train_dataloader,
+                desc=f"Running Epoch {epoch_number} of {args.num_train_epochs}",
+                disable=args.silent,
+                mininterval=0,
+            )
+            for step, batch in enumerate(batch_iterator):
                 if steps_trained_in_current_epoch > 0:
                     steps_trained_in_current_epoch -= 1
                     continue
@@ -551,7 +593,9 @@ class LanguageModelingModel:
                 current_loss = loss.item()
 
                 if show_running_loss:
-                    print("\rRunning loss: %f" % loss, end="")
+                    batch_iterator.set_description(
+                        f"Epochs {epoch_number}/{args.num_train_epochs}. Running Loss: {current_loss:9.4f}"
+                    )
 
                 if args.gradient_accumulation_steps > 1:
                     loss = loss / args.gradient_accumulation_steps
@@ -847,7 +891,8 @@ class LanguageModelingModel:
         if not no_cache:
             no_cache = args.no_cache
 
-        os.makedirs(self.args.cache_dir, exist_ok=True)
+        if not no_cache:
+            os.makedirs(self.args.cache_dir, exist_ok=True)
 
         mode = "dev" if evaluate else "train"
 
@@ -910,7 +955,12 @@ class LanguageModelingModel:
             output_dir = self.args.output_dir
 
         if self.args.model_type in ["bert", "electra"]:
-            tokenizer = BertWordPieceTokenizer()
+            tokenizer = BertWordPieceTokenizer(
+                clean_text=self.args.clean_text,
+                handle_chinese_chars=self.args.handle_chinese_chars,
+                strip_accents=self.args.strip_accents,
+                lowercase=self.args.do_lower_case,
+            )
             self.args.special_tokens = ["[PAD]", "[UNK]", "[CLS]", "[SEP]", "[MASK]"]
             self.args.wordpieces_prefix = "##"
 
@@ -922,7 +972,7 @@ class LanguageModelingModel:
                 wordpieces_prefix="##",
             )
         else:
-            tokenizer = ByteLevelBPETokenizer()
+            tokenizer = ByteLevelBPETokenizer(lowercase=self.args.do_lower_case)
 
             tokenizer.train(
                 files=train_files,
@@ -933,7 +983,7 @@ class LanguageModelingModel:
 
         os.makedirs(output_dir, exist_ok=True)
 
-        tokenizer.save(output_dir)
+        tokenizer.save_model(output_dir)
         logger.info(" Training of {} tokenizer complete. Saved to {}.".format(tokenizer_name, output_dir))
 
         _, _, tokenizer_class = MODEL_CLASSES[self.args.model_type]
@@ -1062,3 +1112,6 @@ class LanguageModelingModel:
         even when training on multiple machines.
         """
         return self.args.local_rank == -1 or torch.distributed.get_rank() == 0
+
+    def get_named_parameters(self):
+        return [n for n, p in self.model.named_parameters()]
