@@ -24,6 +24,7 @@ from simpletransformers.ner.ner_utils import (
     get_examples_from_df,
     get_labels,
     read_examples_from_file,
+    LazyNERDataset,
 )
 from tensorboardX import SummaryWriter
 from torch.nn import CrossEntropyLoss
@@ -49,6 +50,9 @@ from transformers import (
     LongformerConfig,
     LongformerForTokenClassification,
     LongformerTokenizer,
+    MobileBertConfig,
+    MobileBertTokenizer,
+    MobileBertForTokenClassification,
     RobertaConfig,
     RobertaForTokenClassification,
     RobertaTokenizer,
@@ -265,16 +269,58 @@ class NERModel:
         t_total = len(train_dataloader) // args.gradient_accumulation_steps * args.num_train_epochs
 
         no_decay = ["bias", "LayerNorm.weight"]
-        optimizer_grouped_parameters = [
-            {
-                "params": [p for n, p in model.named_parameters() if not any(nd in n for nd in no_decay)],
-                "weight_decay": args.weight_decay,
-            },
-            {
-                "params": [p for n, p in model.named_parameters() if any(nd in n for nd in no_decay)],
-                "weight_decay": 0.0,
-            },
-        ]
+
+        optimizer_grouped_parameters = []
+        custom_parameter_names = set()
+        for group in self.args.custom_parameter_groups:
+            params = group.pop("params")
+            custom_parameter_names.update(params)
+            param_group = {**group}
+            param_group["params"] = [p for n, p in model.named_parameters() if n in params]
+            optimizer_grouped_parameters.append(param_group)
+
+        for group in self.args.custom_layer_parameters:
+            layer_number = group.pop("layer")
+            layer = f"layer.{layer_number}."
+            group_d = {**group}
+            group_nd = {**group}
+            group_nd["weight_decay"] = 0.0
+            params_d = []
+            params_nd = []
+            for n, p in model.named_parameters():
+                if n not in custom_parameter_names and layer in n:
+                    if any(nd in n for nd in no_decay):
+                        params_nd.append(p)
+                    else:
+                        params_d.append(p)
+                    custom_parameter_names.add(n)
+            group_d["params"] = params_d
+            group_nd["params"] = params_nd
+
+            optimizer_grouped_parameters.append(group_d)
+            optimizer_grouped_parameters.append(group_nd)
+
+        if not self.args.train_custom_parameters_only:
+            optimizer_grouped_parameters.extend(
+                [
+                    {
+                        "params": [
+                            p
+                            for n, p in model.named_parameters()
+                            if n not in custom_parameter_names and not any(nd in n for nd in no_decay)
+                        ],
+                        "weight_decay": args.weight_decay,
+                    },
+                    {
+                        "params": [
+                            p
+                            for n, p in model.named_parameters()
+                            if n not in custom_parameter_names and any(nd in n for nd in no_decay)
+                        ],
+                        "weight_decay": 0.0,
+                    },
+                ]
+            )
 
         warmup_steps = math.ceil(t_total * args.warmup_ratio)
         args.warmup_steps = warmup_steps if args.warmup_steps == 0 else args.warmup_steps
@@ -338,9 +384,13 @@ class NERModel:
                 epochs_trained -= 1
                 continue
             train_iterator.set_description(f"Epoch {epoch_number + 1} of {args.num_train_epochs}")
-            for step, batch in enumerate(
-                tqdm(train_dataloader, desc=f"Running Epoch {epoch_number}", disable=args.silent)
-            ):
+            batch_iterator = tqdm(
+                train_dataloader,
+                desc=f"Running Epoch {epoch_number} of {args.num_train_epochs}",
+                disable=args.silent,
+                mininterval=0,
+            )
+            for step, batch in enumerate(batch_iterator):
                 if steps_trained_in_current_epoch > 0:
                     steps_trained_in_current_epoch -= 1
                     continue
@@ -358,7 +408,9 @@ class NERModel:
                 current_loss = loss.item()
 
                 if show_running_loss:
-                    print("\rRunning loss: %f" % loss, end="")
+                    batch_iterator.set_description(
+                        f"Epochs {epoch_number}/{args.num_train_epochs}. Running Loss: {current_loss:9.4f}"
+                    )
 
                 if args.gradient_accumulation_steps > 1:
                     loss = loss / args.gradient_accumulation_steps
@@ -898,66 +950,70 @@ class NERModel:
             no_cache = args.no_cache
 
         mode = "dev" if evaluate else "train"
-
-        if not to_predict:
-            if isinstance(data, str):
-                examples = read_examples_from_file(data, mode)
+        if not to_predict and isinstance(data, str) and self.args.lazy_loading:
+            dataset = LazyNERDataset(data, tokenizer, self.args)
+        else:
+            if not to_predict:
+                if isinstance(data, str):
+                    examples = read_examples_from_file(data, mode)
+                else:
+                    if self.args.lazy_loading:
+                        raise ValueError("Input must be given as a path to a file when using lazy loading")
+                    examples = get_examples_from_df(data)
             else:
-                examples = get_examples_from_df(data)
-        else:
-            examples = to_predict
-            no_cache = True
+                examples = to_predict
+                no_cache = True
 
-        cached_features_file = os.path.join(
-            args.cache_dir,
-            "cached_{}_{}_{}_{}_{}".format(
-                mode, args.model_type, args.max_seq_length, self.num_labels, len(examples),
-            ),
-        )
-        if not no_cache:
-            os.makedirs(self.args.cache_dir, exist_ok=True)
-
-        if os.path.exists(cached_features_file) and (
-            (not args.reprocess_input_data and not no_cache)
-            or (mode == "dev" and args.use_cached_eval_features and not no_cache)
-        ):
-            features = torch.load(cached_features_file)
-            logger.info(f" Features loaded from cache at {cached_features_file}")
-        else:
-            logger.info(" Converting to features started.")
-            features = convert_examples_to_features(
-                examples,
-                self.args.labels_list,
-                self.args.max_seq_length,
-                self.tokenizer,
-                # XLNet has a CLS token at the end
-                cls_token_at_end=bool(args.model_type in ["xlnet"]),
-                cls_token=tokenizer.cls_token,
-                cls_token_segment_id=2 if args.model_type in ["xlnet"] else 0,
-                sep_token=tokenizer.sep_token,
-                # RoBERTa uses an extra separator b/w pairs of sentences,
-                # cf. github.com/pytorch/fairseq/commit/1684e166e3da03f5b600dbb7855cb98ddfcd0805
-                sep_token_extra=bool(args.model_type in ["roberta"]),
-                # PAD on the left for XLNet
-                pad_on_left=bool(args.model_type in ["xlnet"]),
-                pad_token=tokenizer.convert_tokens_to_ids([tokenizer.pad_token])[0],
-                pad_token_segment_id=4 if args.model_type in ["xlnet"] else 0,
-                pad_token_label_id=self.pad_token_label_id,
-                process_count=process_count,
-                silent=args.silent,
-                use_multiprocessing=args.use_multiprocessing,
-                chunksize=args.multiprocessing_chunksize,
+            cached_features_file = os.path.join(
+                args.cache_dir,
+                "cached_{}_{}_{}_{}_{}".format(
+                    mode, args.model_type, args.max_seq_length, self.num_labels, len(examples),
+                ),
             )
-
             if not no_cache:
-                torch.save(features, cached_features_file)
+                os.makedirs(self.args.cache_dir, exist_ok=True)
 
-        all_input_ids = torch.tensor([f.input_ids for f in features], dtype=torch.long)
-        all_input_mask = torch.tensor([f.input_mask for f in features], dtype=torch.long)
-        all_segment_ids = torch.tensor([f.segment_ids for f in features], dtype=torch.long)
-        all_label_ids = torch.tensor([f.label_ids for f in features], dtype=torch.long)
+            if os.path.exists(cached_features_file) and (
+                (not args.reprocess_input_data and not no_cache)
+                or (mode == "dev" and args.use_cached_eval_features and not no_cache)
+            ):
+                features = torch.load(cached_features_file)
+                logger.info(f" Features loaded from cache at {cached_features_file}")
+            else:
+                logger.info(" Converting to features started.")
+                features = convert_examples_to_features(
+                    examples,
+                    self.args.labels_list,
+                    self.args.max_seq_length,
+                    self.tokenizer,
+                    # XLNet has a CLS token at the end
+                    cls_token_at_end=bool(args.model_type in ["xlnet"]),
+                    cls_token=tokenizer.cls_token,
+                    cls_token_segment_id=2 if args.model_type in ["xlnet"] else 0,
+                    sep_token=tokenizer.sep_token,
+                    # RoBERTa uses an extra separator b/w pairs of sentences,
+                    # cf. github.com/pytorch/fairseq/commit/1684e166e3da03f5b600dbb7855cb98ddfcd0805
+                    sep_token_extra=bool(args.model_type in ["roberta"]),
+                    # PAD on the left for XLNet
+                    pad_on_left=bool(args.model_type in ["xlnet"]),
+                    pad_token=tokenizer.convert_tokens_to_ids([tokenizer.pad_token])[0],
+                    pad_token_segment_id=4 if args.model_type in ["xlnet"] else 0,
+                    pad_token_label_id=self.pad_token_label_id,
+                    process_count=process_count,
+                    silent=args.silent,
+                    use_multiprocessing=args.use_multiprocessing,
+                    chunksize=args.multiprocessing_chunksize,
+                )
 
-        dataset = TensorDataset(all_input_ids, all_input_mask, all_segment_ids, all_label_ids)
+                if not no_cache:
+                    torch.save(features, cached_features_file)
+
+            all_input_ids = torch.tensor([f.input_ids for f in features], dtype=torch.long)
+            all_input_mask = torch.tensor([f.input_mask for f in features], dtype=torch.long)
+            all_segment_ids = torch.tensor([f.segment_ids for f in features], dtype=torch.long)
+            all_label_ids = torch.tensor([f.label_ids for f in features], dtype=torch.long)
+
+            dataset = TensorDataset(all_input_ids, all_input_mask, all_segment_ids, all_label_ids)
 
         return dataset
 
@@ -1023,3 +1079,6 @@ class NERModel:
         args = NERArgs()
         args.load(input_dir)
         return args
+
+    def get_named_parameters(self):
+        return [n for n, p in self.model.named_parameters()]
